import pandas as pd
import numpy as np
import pickle
import sys
from sklearn.metrics import classification_report


# Initialise the important variables needed which include files and example and test start time used in the rehoboam simulation file. 
resultsFileName = "output/second.pkl"
spikeTimeFile = "output/spikestime.txt"
testCsvFile = "output/test.csv"
# i had to initialize the exampleTime and testStartTime variables manually because the %store syntax will only work in ipython notebooks.
noTestItem = 203 # you need to change this value depending on whether the data is upsampled (203) or downsampled (48)
#noTestItem = 48
exampleTime = 30.0 # This needs to be changed aswell depending on the EXAMPLETIME set in rehoboam.py
testStartTime = 56740 # you need to get this value from the rehoboam output. The value of END_OF_TRAINING.
#testStartTime = 13420
#%store -r exampleTime
#%store -r testStartTime
#print('Example Time =', exampleTime)
#print('Test start Time =',  testStartTime)


# Unpickle the files and save the output neuron spike times to a txt file.
def printPklSpikes(fileName):
        fileHandle = open(fileName, 'rb')
        fileWrite = open(spikeTimeFile, 'w')
        neoObj = pickle.load(fileHandle)
        segments = neoObj.segments
        segment = segments[0]
        spikeTrains = segment.spiketrains
        neurons = len(spikeTrains)
        for neuronNum in range (0,neurons):
            if (len(spikeTrains[neuronNum])>0):
                spikes = spikeTrains[neuronNum]
                for spike in range (0,len(spikes)):
                    print(neuronNum, spikes[spike], file=fileWrite)
        fileHandle.close()
        fileWrite.close()


# Read the txt file and takes only test spike times.
def readResultsFile(testStartTime):
    testTimeResults = [[],[],[],[]]
    fileHandle = open(spikeTimeFile) 
    data = fileHandle.readlines()
    for i in range(0, len(data)):
        dataLine = data[i]
        stripData = dataLine.strip()
        splitData = stripData.split()
        neuronNumber = int(splitData[0])
        spikeTime = float(splitData[1])
        if (spikeTime >= testStartTime):
            testTimeResults[neuronNumber] = testTimeResults[neuronNumber]+ [spikeTime]
    fileHandle.close()
    return testTimeResults


# This method counts the test spike times for each output neuron and stores them in a list of arrays.
def convertTestSpikeTimesToPerExample(testSpikeTimes,testStartTime,exampleTime):
    noTestItems = noTestItem
    answer0 = np.zeros(noTestItems)
    for spikeTimeOffset in range (0,len(testSpikeTimes[0])):
        actualSpikeTime = testSpikeTimes[0][spikeTimeOffset]
        baseTime = actualSpikeTime-testStartTime
        exampleOffset = baseTime/exampleTime
        exampleOffset = int (exampleOffset)
        if (exampleOffset >= 0) and (exampleOffset < noTestItems):
            answer0[exampleOffset] = answer0[exampleOffset] + 1
        elif (exampleOffset >= noTestItems):
            print('firing beyond end in cat 0', actualSpikeTime)
            

    answer1 = np.zeros(noTestItems)
    for spikeTimeOffset in range (0,len(testSpikeTimes[1])):
        actualSpikeTime = testSpikeTimes[1][spikeTimeOffset]
        baseTime = actualSpikeTime-testStartTime
        exampleOffset = baseTime/exampleTime
        exampleOffset = int (exampleOffset)
        #print actualSpikeTime,baseTime,exampleOffset
        if (exampleOffset >= 0) and (exampleOffset < noTestItems):
            answer1[exampleOffset] = answer1[exampleOffset] + 1
        elif (exampleOffset >= noTestItems):
            print('firing beyond end in cat 1', actualSpikeTime)
            

    answer2 = np.zeros(noTestItems)
    for spikeTimeOffset in range (0,len(testSpikeTimes[2])):
        actualSpikeTime = testSpikeTimes[2][spikeTimeOffset]
        baseTime = actualSpikeTime-testStartTime
        exampleOffset = baseTime/exampleTime
        exampleOffset = int (exampleOffset)
        #print actualSpikeTime,baseTime,exampleOffset
        if (exampleOffset >= 0) and (exampleOffset < noTestItems):
            answer2[exampleOffset] = answer2[exampleOffset] + 1
        elif (exampleOffset >= noTestItems):
            print('firing beyond end in cat 2', actualSpikeTime)
    
    answer3 = np.zeros(noTestItems)
    for spikeTimeOffset in range (0,len(testSpikeTimes[3])):
        actualSpikeTime = testSpikeTimes[3][spikeTimeOffset]
        baseTime = actualSpikeTime-testStartTime
        exampleOffset = baseTime/exampleTime
        exampleOffset = int (exampleOffset)
        #print actualSpikeTime,baseTime,exampleOffset
        if (exampleOffset >= 0) and (exampleOffset < noTestItems):
            answer3[exampleOffset] = answer3[exampleOffset] + 1
        elif (exampleOffset >= noTestItems):
            print('firing beyond end in cat 3', actualSpikeTime)

    spikesPerExample = [answer0,answer1,answer2,answer3]
    return spikesPerExample


# This method uses the maximum argument rule to convert the spike counts to answers.
def spikesConvert(spikesPerExample):
    answers = []
    answers = np.argmax(spikesPerExample, axis = 0) + 1
    return answers


# This method gets the actual answers from the test data and saves it to an array for comparisons and prediction accuracy calculation.
def getGoldStandardAnswers(testFile):
    testData = pd.read_csv(testFile)
    answers = np.array(testData['target'])
    return answers

# This method was used to compare answers from the different rules.
def spikeCompare(spikes, initialAnswers, goldAnswers):
    compareDf = pd.DataFrame()
    compareDf['one'] = spikes[0]
    compareDf['two'] = spikes[1]
    compareDf['three'] = spikes[2]
    compareDf['four'] = spikes[3]
    compareDf['sys'] = initialAnswers
    compareDf['gold'] = goldAnswers

    
    return compareDf


# This is used to calulate the prediction accuracy of the data
def results (systemAnswers,goldAnswers):
    correctAnswers = 0
    wrongAnswers  = 0
    accuracy = 0
    for answerOffset in range (0, noTestItem):
        if (systemAnswers[answerOffset] == goldAnswers[answerOffset]):
            correctAnswers = correctAnswers + 1
        elif (systemAnswers[answerOffset] != goldAnswers[answerOffset]):
            wrongAnswers = wrongAnswers + 1
        #print (systemAnswers[answerOffset] , goldAnswers[answerOffset])

    accuracy = round(correctAnswers/noTestItem * 100, 1)
    print ("Accuracy: {}% ".format(accuracy))


#Get the spikes
printPklSpikes(resultsFileName)


#Store the test spike times
testSpikeTimes = readResultsFile(testStartTime)
#print(testSpikeTimes)

#Convert the test Spike Times and store
spikesPerExample = convertTestSpikeTimesToPerExample(testSpikeTimes,testStartTime,exampleTime)

#print(spikesPerExample)

#Convert spikes to answers with maximum argument rule and store
initialAnswers = spikesConvert(spikesPerExample)

#Store the actual test data answers
actualAnswers = getGoldStandardAnswers(testCsvFile)

#Compare the answers and check spikes for patterns
print(spikeCompare(spikesPerExample, initialAnswers, actualAnswers))

#Maximum argument Accuracy
print(results(initialAnswers, actualAnswers))

#Print Classification Report
print(classification_report(actualAnswers, initialAnswers))
